****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_qrpedenom.sas  
*
* Created (mm/dd/yyyy): 03/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  This macro will compute denominators.                                        
*   
*  Program inputs:                                                                                   
*  	-Many datasets created by ms_qrpenum
* 
*  Program outputs:                                                                                                                                       
*  	-The DPLocal.&RUNID._DenomCounts output table
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*		
***************************************************************************************************;


%macro ms_qrpedenom();

%put =====> MACRO CALLED: ms_qrpedenom v1.0;

/*---------------------------------------------------*/
/* Original encounters								 */
/*---------------------------------------------------*/

	proc sort data=ENC_&GROUP. out=_denom&group. nodupkey;
		by patid encounterID;
	run;

/*-------------------------------------------------------*/
/* Inclusion/Exclusion required (POV3) for eligibility	 */
/*-------------------------------------------------------*/  

	data _denom&group.;
		merge _denom&group.(in=a)
			  _pov3(in=b);
		by patid encounterID;
		if a and b;

		drop birth_date sex race hispanic;
	run;
		
/*---------------------------*/
/* Add DOB/SEX/Race/Hispanic */
/*---------------------------*/

	proc sort nodupkey data=_Denom&Group. out=_uniquePts(Keep=PatId);
		by PatId;
	run;

    proc sql noprint;
        /*Age and demogs*/		
		create table _DemoCount as
        select Dem.PatId, 
			   Count(Dem.PatId) as numdemo
        from indata.&demtable. as DEM 
             inner join 
             _UniquePts as pts
        on DEM.patid = pts.patid
		group by Dem.PatId;

        *restrict to unique pts;
        create table _Demo as
        select distinct pts.*, 
                        birth_date, 
						case when sex = 'F' then 'F'
							 when sex = 'M' then 'M'
							 else 'O'						
						end as sex,
						race,
						hispanic
        from indata.&demtable as DEM 
             inner join 
             _Denom&Group. as pts			 
        on DEM.patid = pts.patid
			 inner join _DemoCount (where=(numdemo=1)) as cnt
		on cnt.patid = pts.patid;

		create table  _Denom&Group.
		as select * 
		from _demo;        
    quit;

	*truncate periods such that all interval lies in Minage-MaxAge;

    data _Denom&Group.;
    	set _Denom&Group.;

	    format MinAgeDate  MaxAgeDate date9.;
	    MinAgeDate = intnx(scan("&AGETYP.",1),birth_date,&MINAGE.,'sameday');

	    if &MAXAGE.=99999 then MaxAgeDate=intnx('Years',birth_date,110,'sameday');
	    else MaxAgeDate= intnx(scan("&AGETYP.",&NBSTRAT.),birth_date,&MAXAGE.+1,'sameday')-1; 

	    *Encounter not meetin minimum age yet;
	    if MinAgeDate > adate then delete;

	    *Member has reached or will reach will reach maxage ;
	    if MaxAgeDate < adate then delete;

		encounter = 1;
		facility = 1;
		patient = 1;

		year =year(adate);

		*Combine IP and IS into IP;
		if enctype = 'IS' then enctype = 'IP';

		*Reclassify missing discharge_disposition as U;
		if missing(discharge_disposition) then discharge_disposition='U';
    run;

	*calculate agegroup;
	%ms_agestrat(infile=_Denom&Group.,
                 outfile=_Denom&Group.,
                 startdt=birth_date,
                 enddt=adate,
                 timestrat=&agestrat.);

	/*************************************/
	/* Calculate and output denominators */
	/*************************************/

	proc datasets lib=work nowarn noprint;
		delete _denomcounts:;
	quit;

	*Create Overall counts;
	%macro createalldenumcounts(metric=, class=, outvar=);
	    proc means data=_Denom&Group. nway noprint;
	    	var &metric.;
	    	class &class.;
	    	output out=_ALL_&outvar. max(&metric.)=;
	    run;
	    proc means data=_ALL_&outvar. noprint;
		    var &metric.;
		    output out=_ALL_&outvar.(drop=_:) sum(&metric.)=DenNum&outvar.;
	    run;
	%mend;

	%createalldenumcounts(metric=Encounter, class =encounterID, outvar=Enc);
	%createalldenumcounts(metric=Patient, class =patID, outvar=Pt);
	%createalldenumcounts(metric=Facility, class =facility_code, outvar=Fac);

	data _denomcounts;
		set _all_enc 
			_all_pt
			_all_fac;

	        format level $3. group $30.;
	        Level="000";
			group = "&ITGROUP.";
	run;

	*create stratified counts;
	%macro createstratdenumcounts(metric=, class=, strata=, outvar=, level=);
	    proc sort data=_Denom&Group.;
		    by &strata. &class.;
	    run;

		proc means data=_Denom&Group. noprint;
		    var &metric.;
		    by &strata. &class.;
		    output out=_&strata._&outvar. max(&metric.)=;
			where &strata. is not missing;
	    run;
	    proc means data=_&strata._&outvar. noprint;
		    var &metric.;
		    by &strata.;
		    output out=_&strata._&outvar.(drop=_:) sum(&metric.)=DenNum&outvar.;
	    run;

		data _denomcounts;
	    	set  _denomcounts
	             _&strata._&outvar.(in=a);
	        if a then do;	
				Level="&level.";
				group = "&ITGROUP.";
			end;
		run;
	%mend;

	*year;
	%createstratdenumcounts(metric=Encounter, class=encounterID, strata=year, outvar=Enc, level=001);
	%createstratdenumcounts(metric=Patient, class=patid, strata=year, outvar=Pt, level=001);
	%createstratdenumcounts(metric=Facility, class=facility_code, strata=year, outvar=Fac, level=001);

	*sex;
	%createstratdenumcounts(metric=Encounter, class=encounterID, strata=sex, outvar=Enc, level=002);
	%createstratdenumcounts(metric=Patient, class=patid, strata=sex, outvar=Pt, level=002);
	%createstratdenumcounts(metric=Facility, class=facility_code, strata=sex, outvar=Fac, level=002);

	*age group;
	%createstratdenumcounts(metric=Encounter, class=encounterID, strata=agegroup, outvar=Enc, level=003);
	%createstratdenumcounts(metric=Patient, class=patid, strata=agegroup, outvar=Pt, level=003);
	%createstratdenumcounts(metric=Facility, class=facility_code, strata=agegroup, outvar=Fac, level=003);

	*race;
	%createstratdenumcounts(metric=Encounter, class=encounterID, strata=race, outvar=Enc, level=110);
	%createstratdenumcounts(metric=Patient, class=patid, strata=race, outvar=Pt, level=110);
	%createstratdenumcounts(metric=Facility, class=facility_code, strata=race, outvar=Fac, level=110);

	*hispanic;
	%createstratdenumcounts(metric=Encounter, class=encounterID, strata=hispanic, outvar=Enc, level=115);
	%createstratdenumcounts(metric=Patient, class=patid, strata=hispanic, outvar=Pt, level=115);
	%createstratdenumcounts(metric=Facility, class=facility_code, strata=hispanic, outvar=Fac, level=115);

	*EncType;
	%createstratdenumcounts(metric=Encounter, class=encounterID, strata=enctype, outvar=Enc, level=200);
	%createstratdenumcounts(metric=Patient, class=patid, strata=enctype, outvar=Pt, level=200);
	%createstratdenumcounts(metric=Facility, class=facility_code, strata=enctype, outvar=Fac, level=200);

	*Discharge Disposition.;
	%createstratdenumcounts(metric=Encounter, class=encounterID, strata=discharge_disposition, outvar=Enc, level=205);
	%createstratdenumcounts(metric=Patient, class=patid, strata=discharge_disposition, outvar=Pt, level=205);
	%createstratdenumcounts(metric=Facility, class=facility_code, strata=discharge_disposition, outvar=Fac, level=205);

	**collapse to 1 row per level/strata value;
	proc means data=_denomcounts noprint missing nway;
		var DenNumEnc DenNumPt DenNumFac;
		class group level sex agegroup year race hispanic enctype discharge_disposition;
		output out=_DenomCounts_agg(drop=_:) max=;
	run;


	**store in dplocal**;
    %IF %EVAL(&group.=1) %THEN %DO;
        data dplocal.&runid._DenomCounts;
        	retain Level group sex agegroup year race hispanic enctype discharge_disposition DenNumEnc DenNumPt DenNumFac;
        	set _DenomCounts_agg;
        run;
    %END;
    %ELSE %DO;
        proc append base=dplocal.&runid._DenomCounts 
                    data=_DenomCounts_agg force;
        run;
    %END;

	proc datasets nowarn noprint lib=work;
		delete _all_: _sex_: _year: _agegroup_: _race_: _hispanic_: _enctype_: _discharge_disposition_:;
	quit;
		
%put NOTE: ******** END OF MACRO: ms_qrpedenom v1.0 ********;

%mend ms_qrpedenom;
