***************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_qrpenum.sas  
*
* Created (mm/dd/yyyy): 02/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will compute the numerators and call many other macros creating the denominators 
*	and the output tables.                                        
*   
*  Program inputs:    
*	-infolder.&MONITORINGFILE.
*	-infolder.&COHORTFILE.
*	-infolder.&COHORTCODES. 
*	-infolder.&INCLUSIONCODES. 
*	-infolder.&COVARIATECODES. 
*   -infolder.&FACILITYCODES.  
*   -infolder.&TRANSCODE_TYPE_LOOKUP.  
*
*  Program outputs:       
*                                                                                                  
*  PARAMETERS:  
*	-RUNID					= Run identifier to denote each execution of the program
*	-MSPROJID				= MSOC-defined value  
*	-MSWPTYPE				= MSOC-defined value
*	-MSWPID					= MSOC-defined value
*	-MSDPID					= MSOC-defined value
*	-MSVERID				= MSOC-defined value
*	-MONITORNGFILE			= Name of the SAS dataset that defines the relevant query periods for execution
*	-COHORTFILE				= Name of the SAS dataset that defines who is in the cohort (demographic criteria) 
*	-COHORTCODES			= Name of the SAS dataset that includes the codes defining the cohort 
*	-INCLUSIONCODES			= Name of the SAS dataset that includes the codes defining inclusions/exclusions
*	-COVARIATECODES			= Name of the SAS dataset that includes the codes associated with covariates 
*	-FACILITYCODES			= Name of the SAS dataset that includes facility codes to include
*	-TRANSCODE_TYPE_LOOKUP	= Name of the SAS dataset that defines blood components
*	-FREEZEDATA				= Indicates if MSDD patient data shoul be copied to DPLocal
*
*  Programming Notes:                                                                                
*                                                                           
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_qrpenum(RUNID,
                  MSPROJID,  
                  MSWPTYPE,
                  MSWPID,
                  MSDPID,
                  MSVERID,
                  MONITORINGFILE,
                  COHORTFILE,
                  COHORTCODES,
                  INCLUSIONCODES,
                  COVARIATECODES,
				  FACILITYCODES,
				  TRANSCODE_TYPE_LOOKUP,
				  FREEZEDATA);

	%put =====> MACRO CALLED: ms_qrpenum v1.0;

	%ms_starttimer(START=MAINSTART);

	/*---------------------------------------------------*/
	/*  01 -- Set up variables needed for the run        */
	/*---------------------------------------------------*/
	%LET MSPROJID=%LOWCASE(&MSPROJID.);
	%LET MSWPTYPE=%LOWCASE(&MSWPTYPE.);
	%LET MSWPID=%LOWCASE(&MSWPID.);
	%LET MSDPID=%LOWCASE(&MSDPID.);
	%LET MSVERID=%LOWCASE(&MSVERID.);

	%LET MONITORINGFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&MONITORINGFILE.), .sas7bdat, %STR())));
	%LET COHORTFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COHORTFILE.), .sas7bdat, %STR())));
	%LET COHORTCODES=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COHORTCODES.), .sas7bdat, %STR())));
	%LET INCLUSIONCODES=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&INCLUSIONCODES.), .sas7bdat, %STR())));
	%LET COVARIATECODES=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COVARIATECODES.), .sas7bdat, %STR())));
	%LET FACILITYCODES=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&FACILITYCODES.), .sas7bdat, %STR())));
	%LET TRANSCODE_TYPE_LOOKUP=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TRANSCODE_TYPE_LOOKUP.), .sas7bdat, %STR())));

	/*---------------------------------------------------*/
	/*  02 -- Import input files in SAS format           */
	/*---------------------------------------------------*/

	%IMPORTFILES(var=&MONITORINGFILE.);
	%IMPORTFILES(var=&COHORTFILE.);
	%IMPORTFILES(var=&COHORTCODES.);
	%IMPORTFILES(var=&INCLUSIONCODES.);
	%IMPORTFILES(var=&COVARIATECODES.);
	%IMPORTFILES(var=&FACILITYCODES.);
	%IMPORTFILES(var=&TRANSCODE_TYPE_LOOKUP.);
	%IMPORTFILES(var=&PTSTOEXCLUDE.);

	/*---------------------------------------------------*/
	/*  03 -- Initialize macro variables and 			 */
	/*        Process input file parameters              */
	/*---------------------------------------------------*/

	*Initialize macro variables;
	%ms_qrpe_initmacrovariables();

	*Process input files;
	%ms_qrpe_processinputfiles();

	/*---------------------------------------------------*/
	/*  04 -- Process Ranges "-" and Wilcards "*"        */
	/*        in lookup codes                            */
	/*---------------------------------------------------*/

	%ISDATA(dataset=_diag);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;
	    %MS_ProcessWildcards(InFile=_diag, 
	                         CodeVar=code, 
	                         OutFile=_diag);
	%END;

	/*------------------------------*/
	/*  05 -- Extract Raw Data      */
	/*------------------------------*/

	*Extract all encounters during query period;
        proc sql noprint;
	        create table encextract(keep=patid encounterid adate ddate discharge_disposition enctype facility_code) as
	        select claim.*
	        from indata.&enctable. as claim
	        where &startfollowup. <= claim.adate <= &enddate.
			order by patid, encounterid;
        quit;

	 	*Delete data from patients in DPs &PTSTOEXCLUDE. files; 
   		%ms_delpatients(datafile=encextract,
                   ptsfile=&PTSTOEXCLUDE.,
                   Outfile=encextract);


	*Programming note:  We are not using caresettingprincipal in the pre-extraction phase because
     typical sensitivity analysis around caresettingprincipal could lead to massive claim duplication;

	*Extract procedures;
		%ms_extractmeds(datafile=indata.&proctable.,
                   datacode=PX ,
                   datacodetype=PX_codetype ,
                   lookfile=_proc, 
                   outfile=_procextract);

		proc sort data=_procextract nodupkey;
	   		by _all_;
	   	run;

	   	*Delete data from patients in DPs &PTSTOEXCLUDE. files; 
	   	%ms_delpatients(datafile=_procextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=_procextract);

	*Extract diagnoses;
   		%ms_extractmeds(datafile=indata.&diatable.,
                   datacode=DX,
                   datacodetype=DX_codetype,
                   lookfile=_diag, 
                   outfile=_diagextract);

		proc sort data=_diagextract nodupkey;
			by _all_;
		run;

	   *Delete data from patients in DPs &PTSTOEXCLUDE. files; 
	   %ms_delpatients(datafile=_diagextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=_diagextract);


	*Extract inpatient-drugs;
	    *Codes must be unique to avoid claim duplication.;
	    proc sort nodupkey data=_inpatndc;
	    	by code;
	    run;

    	%ms_extractinpatientdrugs(datafile=indata.&IPHARMTABLE.,
                     datavar=NDC ,
                     lookfile=_inpatndc(drop=codecat CodeType), 
                     lookvar=code , 
                     outfile=inpatndcextract);

		proc sort data=inpatndcextract nodupkey;
			by _all_;
		run;

	    *Delete data from patients in DPs &PTSTOEXCLUDE. files; 
		%ms_delpatients(datafile=inpatndcextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=inpatndcextract);
                 
	*Extract inpatient-transfusions;
		%ms_extractmeds(datafile=indata.&ITRANSTABLE.,
                   datacode=transcode ,
                   datacodetype=Transcode_type ,
                   lookfile=_trans, 
                   outfile=transextract);

		proc sort data=transextract nodupkey;
			by _all_;
		run;

	   *Delete data from patients in DPs &PTSTOEXCLUDE. files; 
	   %ms_delpatients(datafile=transextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=transextract);

		*Add additional vars;
		data transextract;
			set transextract;
			padmit = ' ';
			PDX = ' ';
			code = compress(transcode, ' .');
			rename tdate_start = adate;
			rename tdate_end = ExpireDt;
		run;

	*Combine _diags and _procs;
	proc sql noprint;
		create table meds as 
		select proc.PatId,
			   proc.encounterID,
		       proc.Adate,
		       compress(proc.PX,' .')as Code,
		       "  " as Pdx,
			   " " as Padmit,
		       proc.EncType,
		       PX_Codetype as Codetype,
		       "PX" as CodeCat,
			   proc.adate as ExpireDt
		from _procextract as proc
		union corr all
		select diag.PatId,
			   diag.encounterID,
		       diag.Adate,
		       compress(diag.DX,' .') as Code,
		       diag.EncType,
		       diag.Pdx,
			   diag.padmit,
		       diag.DX_Codetype as Codetype,
		       "DX" as CodeCat,
			   diag.adate as ExpireDt
		from _diagextract as diag;
	quit; 

	*To avoid w.a.r.n.i.n.g.s. down the line;
	data meds;
		set meds;
		length Adate 4.;
	run;

	proc datasets library=work nowarn nolist;
		delete _:;
	quit;

	/*---------------------------------------------------*/
	/*  06 -- Loop through Group(s)			             */ 
	/*---------------------------------------------------*/

	%macro forloop;

	%do Group=1 %to &Ngroup.;

	/*---------------------------------------------------*/
	/*  06.a -- Loop pre-processing                      */ 
	/*---------------------------------------------------*/
		
	    *Set current Group macro parameters;
		%ms_qrpe_setloopmacrovars();

		%PUT INFO: NOW LOOPING ON &ITGROUP.;

	    *Get specific codes for this loop and separate diags from others to process wildcards;
	    data _diag 
	         _nondiag;
	    set &COHORTCODES.;
	    where group="&ITGROUP.";
		code=upcase(code);
	    if CodeCat in("DX") then output _diag ;
	    else output _nondiag;
	    run;

	    %ISDATA(dataset=_diag);
	    %IF %EVAL(&NOBS.>=1) %THEN %DO;
	        %MS_ProcessWildcards(InFile=_diag, 
	                             CodeVar=code, 
	                             OutFile=_diag);
	    %END;

	    *Create Loop-specific parameter files;
	    data /*_ITGroupParamOutPatNDC*/
			 _ITGroupParamInPatNDC
	         _ITGroupParamMeds
			 _ITGroupParamTrans;
		set _diag
		    _nondiag;
		if Condinclusion=. then Condinclusion=1;
		if SubCondinclusion=. then SubCondinclusion=1;
		if Cond=. then Cond=-1;
		if SubCond=. then SubCond=-1;	
		if codedays = . then codedays = 1;
		code=upcase(code);

		/*if CodeCat="DR"          then output _ITGroupParamOutPatNDC;*/
		if CodeCat="IR"          then output _ITGroupParamInPatNDC;
		if CodeCat in("DX","PX") then output _ITGroupParamMeds;
		if CodeCat in("TR")      then output _ITGroupParamTrans;
	    run;

		*Dedup to avoid claim duplication;
		proc sort data=_ITGroupParamInPatNDC nodupkey;
			by _all_;
		run;
 		proc sort data=_ITGroupParamMeds nodupkey;
			by _all_;
		run;		
		proc sort data=_ITGroupParamTrans nodupkey;
			by _all_;
		run;
	         
	    *Break out CaresettingPrincipal into Caresetting and Principal;
	    %ms_caresettingprincipal(InFile=_ITGroupParamMeds, 
	                             Var=CareSettingPrincipal, 
	                             OutFile=_ITGroupParamMeds);

	    %ms_caresettingprincipal(InFile=_ITGroupParamTrans, 
	                             Var=CareSettingPrincipal, 
	                             OutFile=_ITGroupParamTrans);

		*Break of PAdmit;
	    %ms_padmit(InFile=_ITGroupParamMeds, 
	               Var=padmit, 
	               OutFile=_ITGroupParamMeds);
		%ms_padmit(InFile=_ITGroupParamTrans, 
	               Var=padmit, 
	               OutFile=_ITGroupParamTrans);

	/*---------------------------------------------------*/
	/*  06.b -- Extract loop-specific records            */ 
	/*---------------------------------------------------*/

	    *Extract Records necessary for creating this group only
	     (record duplic OK + add loop parameters back to records); 
	    %ms_qrpe_loopmeds(datafile=meds,
	                 lookfile=_ITGroupParamMeds, 
	                 outfile=_ITMeds);

  		%ms_extractinpatientdrugs(datafile=inpatndcextract,
                     datavar=NDC ,
                     lookfile=_ITGroupParamInPatNDC, 
                     lookvar=code , 
                     outfile=_ITInpatDrugs);

	    %ms_qrpe_loopmeds(datafile=transextract,
	                 lookfile=_ITGroupParamTrans, 
	                 outfile=_ITTrans);

	/*------------------------------------------------------------------------------*/
	/*  06.c -- Restrict Encounters to desired demographic criteria and caresetting */ 
	/*------------------------------------------------------------------------------*/    

	 	proc sql noprint ;
			create table ENC_&GROUP. as
			select Enc.*,
				   Dem.BIRTH_DATE,
				   Dem.Sex,
				   Dem.Race,
				   Dem.Hispanic
			from encextract as Enc,
				 indata.&DEMTABLE. as Dem
			where Enc.patid=Dem.patid 
			/*Demographics*/
			and upcase(Dem.sex) in (&DemogSex.) 
			and upcase(Dem.Race) in (&DemogRace.) 
			and upcase(Dem.Hispanic) in (&DemogHispanic.)
			/*Caresetting*/ 
			and upcase(enc.enctype) in (&EncCaresetting.)
			/*Remove encounters based on value of TRUNCYN*/
			%if "&truncyn" = "Y" %then %do;
			and enc.ddate <=&enddate.
			%end;
			order by PatID, encounterID;
		quit;

		* Delete data for patients in DPs &PTSTOEXCLUDE. file; 
		%ms_delpatients(datafile=ENC_&GROUP.,
	                	ptsfile=&PTSTOEXCLUDE.,
	                	Outfile=ENC_&GROUP.);

		* Restrict to desired facilities;
		%isdata(dataset=infolder.&FACILITYCODES.);
		%if %eval(&nobs.>0) %then %do;
			proc sql noprint undo_policy=none;
				create table ENC_&GROUP. as
				select enc.*
				from ENC_&GROUP. as enc
				where enc.facility_code in 
				(select distinct facility_code from infolder.&FACILITYCODES.);
			quit;
		%end;

		* Make sure claims only occur during valid encounters;
		%ms_restricttoencounter(encfile=ENC_&GROUP.,
                       			claimfile=_ITInpatDrugs,
                       			outfile=_ITInpatDrugs);

		%ms_restricttoencounter(encfile=ENC_&GROUP.,
                       			claimfile=_ITMeds,
                       			outfile=_ITMeds);

		%ms_restricttoencounter(encfile=ENC_&GROUP.,
                       			claimfile=_ITTrans,
                       			outfile=_ITTrans);	  

	/*---------------------------------------------------*/
	/*  06.d -- Create raw POVs datasets                 */
	/*---------------------------------------------------*/
	    data _GroupIndex(keep=PatId EncounterID facility_code discharge_disposition enctype Adate expiredt code codetype codecat enc_start enc_end)		
			 _InclExcl                                                          							
			 _FUPEvent(keep=PatId EncounterID FUP);									

	    set _ITInpatDrugs(in=a rename=RxADate=ADate)
	        _ITMeds(in=b)
	        _ITTrans(in=c);

			if a then do;
				Expiredt = Adate;
			end;

	        /*POV1*/ if INDEX in('DEF')   			then output _GroupIndex;
	        /*POV3*/ if INDEX in('INC','EXC')       then output _InclExcl;
	        /*POV5*/ if substr(FUP,1,3) in ('HOI') 	then output _FUPEvent;
	    run;

	/*------------------------------------------------------------*/
	/*  06.e -- POV1: Find all potential encounters with exposure */
	/*------------------------------------------------------------*/

		%ms_qrpe_createpov1();

	/*---------------------------------------------------------------------*/
	/*  06.f -- POV3: identify all encounters meeting inclusion/exclusion  */
	/*---------------------------------------------------------------------*/

		%ms_qrpe_createpov3();    

	/*---------------------------------------------------*/
	/*  06.g -- Create Final Record wrt index date       */
	/*---------------------------------------------------*/

		%ms_createencmasterlist();
		
	/*----------------------------------------------*/
	/*  06.h -- Find HOI within selected encounters */
	/*----------------------------------------------*/

		%ms_qrpe_createpov5();


	/*---------------------------------------------------*/
	/*  06.l -- Finalize Encounters List		         */
	/*---------------------------------------------------*/
	    
		%ms_finalize_encmasterlist();

	/*---------------------------------------------------*/
	/*  06.j -- Store encounters to DPLocal		         */
	/*---------------------------------------------------*/

	%if %eval(&group.=1) %then %do;
		data DPLocal.&RUNID._all_encounter;
       		set _EncMasterList;
        run;
	%end; 
	%else %do;
		proc append base=DPLocal.&RUNID._all_encounter 
                    data=_EncMasterList force;  
		run;
	%end;

	/*-----------------------------*/
	/*  06.k -- Compute attrition  */
	/*-----------------------------*/
	
	 	%ms_qrpe_attrition();

	/*--------------------------------*/
	/*  06.l -- Compute denominators  */
	/*--------------------------------*/

		%ms_qrpedenom();

	/*----------------------------------*/
	/*  06.m -- Compute IPHARM metrics  */
	/*----------------------------------*/

		%ms_rxsummarytable();;

	/*--------------------*/
	/*  06.n -- Clean-up  */
	/*--------------------*/
  	proc datasets library=work nowarn nolist;
	    delete _:;
    quit; 


	* Reset loop macro variables;
	%let EXCL=;
	%let INCL=;

	%put LAST GROUP RAN WAS &ITGROUP. &group.;

	%end; /*group loop*/

	%mend forloop;

	%forloop;

	/************************/
	/* Extract covariates   */
	/************************/

	%ms_qrpecov();

	/************************/
	/* Produce outputs      */
	/************************/

	%ms_qrpetables();

	proc sort data=DPLocal.&RUNID._all_encounter;
		by Group PatId adate;
	run;

	/************************/
	/* Freeze Patients Data */
	/************************/
	%macro wrapper;    
		%IF %UPCASE(%str("&FREEZEDATA."))=%str("Y") %THEN %DO;
			proc sort nodupkey data=DPLocal.&RUNID._all_encounter out=_PtsList(keep=Patid);
				by Patid;
			run;
		
	       %isdata(dataset=_PtsList);
	        %if %eval(&nobs.>0) %then %do;            
	            %IF %STR(&ENRTABLE.) ne %STR() or %LENGTH(&ENRTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&ENRTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&ENRTABLE.);%END;            
	            %IF %STR(&DEMTABLE.) ne %STR() or %LENGTH(&DEMTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DEMTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&DEMTABLE.);%END;            
	            %IF %STR(&DISTABLE.) ne %STR() or %LENGTH(&DISTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DISTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&DISTABLE.);%END;            
	            %IF %STR(&DIATABLE.) ne %STR() or %LENGTH(&DIATABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DIATABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&DIATABLE.);%END;           
	            %IF %STR(&PROCTABLE.) ne %STR() or %LENGTH(&PROCTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&PROCTABLE,PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&PROCTABLE.);%END;            
	            %IF %STR(&ENCTABLE.) ne %STR() or %LENGTH(&ENCTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&ENCTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&ENCTABLE.);%END;           
	            %IF %STR(&LABTABLE.) ne %STR() or %LENGTH(&LABTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&LABTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&LABTABLE.);%END;           
	            %IF %STR(&VITTABLE.) ne %STR() or %LENGTH(&VITTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&VITTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&VITTABLE.);%END;            
	            %IF %STR(&DEATHTABLE.) ne %STR() or %LENGTH(&DEATHTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DEATHTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&DEATHTABLE.);%END;

	            %IF %STR(&IPHARMTABLE.) ne %STR() or %LENGTH(&IPHARMTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&IPHARMTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&IPHARMTABLE.);%END;
	            %IF %STR(&ITRANSTABLE.) ne %STR() or %LENGTH(&ITRANSTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&ITRANSTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._&ITRANSTABLE.);%END;

	            *This datalib will point to the subset of PatId data generated by Freeze data;
	            %let DataLib=DPlocal.&RUNID._;
	        %end;
	    %END;
	%mend wrapper;
	%wrapper;

	proc datasets library=work nowarn nolist kill; quit;

	%ms_stoptimer(START=&MAINSTART.);
 
	*Signature file;
	data signature;
		DPID="&DPID.";
		SITEID="&SITEID.";
		MSReqID="&MSREQID.";
		MSProjID="&MSPROJID.";  
		MSWPType="&MSWPTYPE.";
		MSWPID="&MSWPID.";
		MSDPID="&MSDPID.";
		MSVerID="&MSVERID.";
		RunID="&RUNID.";
		MPNum="QRPE";
		MPVer="&Ver.";

		format StartTime StopTime datetime21.2; 
		StartTime=input("&MAINSTART.",best.); 
		StopTime=input("&STOP.",best.);   
		format Seconds $20.;
		Seconds=put(int(&stop.-&MAINSTART.),best.)||" s";
		ExecutionTime="&hours. h &minutes. m &seconds. s";    
		ScenarioCnt=strip("&Ngroup.");
		MONITORINGFILE="&MONITORINGFILE.";
		COHORTFILE="&COHORTFILE.";
		COHORTCODES="&COHORTCODES.";
		INCLUSIONCODES="&INCLUSIONCODES.";
		COVARIATECODES="&COVARIATECODES.";
		FREEZEDATA="&FREEZEDATA.";
		FACILITYCODES="&FACILITYCODES.";
		TRANSCODE_TYPE_LOOKUP="&TRANSCODE_TYPE_LOOKUP.";
		PATIDEXCL="&PATIDEXCL.";
		ETLNUMBER="&ETL.";
		format DPMINDATE DPMAXDATE date9.;
		DPMINDATE=&DP_MinDate.;
		DPMAXDATE=&DP_MaxDate.;
		output;
	run;

	proc transpose data=signature out=msoc.&RUNID._signature(rename=_NAME_=Var rename=COL1=VALUE);
	   var _ALL_;
	run;

%put NOTE: ********END OF MACRO: ms_qrpenum v1.0********;

%mend ms_qrpenum;
